/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_RMSMMESSAGE_H
#define DPC_IMPL_TYPE_RMSMMESSAGE_H

#include <cfloat>
#include <cmath>
#include "dpc/impl_type_rmheader.h"
#include "dpc/impl_type_array_uint32_10.h"
#include "dpc/impl_type_array_uint8_10.h"
#include "dpc/impl_type_array_leftline_6.h"
#include "dpc/impl_type_array_rightline_6.h"
#include "dpc/impl_type_array_laneinfo_12.h"
#include "dpc/impl_type_array_laneinfo_1.h"
#include "dpc/impl_type_array_leftroadedge_4.h"
#include "dpc/impl_type_array_rightroadedge_4.h"
#include "dpc/impl_type_array_float32_10.h"
#include "dpc/impl_type_array_float64_10.h"
#include "impl_type_uint8.h"
#include "impl_type_float.h"

namespace dpc {
struct RMSMMessage {
    ::dpc::RMHeader rm_header;
    ::dpc::array_uint32_10 rm_crrdr_id{};
    ::dpc::array_uint8_10 rm_featuretag{};
    ::dpc::array_uint32_10 rm_dsblrsn{};
    ::dpc::array_uint32_10 rm_ilc_target{};
    ::dpc::array_LeftLine_6 left_line{};
    ::dpc::array_RightLine_6 right_line{};
    ::dpc::array_LaneInfo_12 left_lane{};
    ::dpc::array_LaneInfo_1 ego_lane{};
    ::dpc::array_LaneInfo_12 right_lane{};
    ::dpc::array_LeftRoadEdge_4 left_roadedge{};
    ::dpc::array_RightRoadEdge_4 right_roadedge{};
    ::dpc::array_float32_10 float_reserve{};
    ::dpc::array_float64_10 double_reserve{};
    ::UInt8 rm_lne_crss{ 0U };
    ::Float rm_lanewidth{ 0.0F };
    ::UInt8 control_way_point_target_road_type{ 0U };
    ::UInt8 spd_lmt_id01{ 0U };
    ::UInt8 control_way_point_speed_limit01{ 0U };
    ::Float control_way_point_offset01{ 0.0F };
    ::UInt8 spd_lmt_id02{ 0U };
    ::UInt8 control_way_point_speed_limit02{ 0U };
    ::Float control_way_point_offset02{ 0.0F };
    ::UInt8 reminder_way_point_targe_road_type{ 0U };
    ::Float reminder_way_point_offset{ 0.0F };
    ::Float remain_distance_major_path{ 0.0F };
    ::UInt8 remain_distance_major_path_type{ 0U };
    ::UInt8 camr_line_mapping{ 0U };
    ::UInt8 location_qulity{ 0U };
    ::Float enter_ramp_dist{ 0.0F };
    ::Float merge_main_road_dist{ 0.0F };
    ::UInt8 inavlts{ 0U };

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(rm_header);
        fun(rm_crrdr_id);
        fun(rm_featuretag);
        fun(rm_dsblrsn);
        fun(rm_ilc_target);
        fun(left_line);
        fun(right_line);
        fun(left_lane);
        fun(ego_lane);
        fun(right_lane);
        fun(left_roadedge);
        fun(right_roadedge);
        fun(float_reserve);
        fun(double_reserve);
        fun(rm_lne_crss);
        fun(rm_lanewidth);
        fun(control_way_point_target_road_type);
        fun(spd_lmt_id01);
        fun(control_way_point_speed_limit01);
        fun(control_way_point_offset01);
        fun(spd_lmt_id02);
        fun(control_way_point_speed_limit02);
        fun(control_way_point_offset02);
        fun(reminder_way_point_targe_road_type);
        fun(reminder_way_point_offset);
        fun(remain_distance_major_path);
        fun(remain_distance_major_path_type);
        fun(camr_line_mapping);
        fun(location_qulity);
        fun(enter_ramp_dist);
        fun(merge_main_road_dist);
        fun(inavlts);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(rm_header);
        fun(rm_crrdr_id);
        fun(rm_featuretag);
        fun(rm_dsblrsn);
        fun(rm_ilc_target);
        fun(left_line);
        fun(right_line);
        fun(left_lane);
        fun(ego_lane);
        fun(right_lane);
        fun(left_roadedge);
        fun(right_roadedge);
        fun(float_reserve);
        fun(double_reserve);
        fun(rm_lne_crss);
        fun(rm_lanewidth);
        fun(control_way_point_target_road_type);
        fun(spd_lmt_id01);
        fun(control_way_point_speed_limit01);
        fun(control_way_point_offset01);
        fun(spd_lmt_id02);
        fun(control_way_point_speed_limit02);
        fun(control_way_point_offset02);
        fun(reminder_way_point_targe_road_type);
        fun(reminder_way_point_offset);
        fun(remain_distance_major_path);
        fun(remain_distance_major_path_type);
        fun(camr_line_mapping);
        fun(location_qulity);
        fun(enter_ramp_dist);
        fun(merge_main_road_dist);
        fun(inavlts);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("rm_header", rm_header);
        fun("rm_crrdr_id", rm_crrdr_id);
        fun("rm_featuretag", rm_featuretag);
        fun("rm_dsblrsn", rm_dsblrsn);
        fun("rm_ilc_target", rm_ilc_target);
        fun("left_line", left_line);
        fun("right_line", right_line);
        fun("left_lane", left_lane);
        fun("ego_lane", ego_lane);
        fun("right_lane", right_lane);
        fun("left_roadedge", left_roadedge);
        fun("right_roadedge", right_roadedge);
        fun("float_reserve", float_reserve);
        fun("double_reserve", double_reserve);
        fun("rm_lne_crss", rm_lne_crss);
        fun("rm_lanewidth", rm_lanewidth);
        fun("control_way_point_target_road_type", control_way_point_target_road_type);
        fun("spd_lmt_id01", spd_lmt_id01);
        fun("control_way_point_speed_limit01", control_way_point_speed_limit01);
        fun("control_way_point_offset01", control_way_point_offset01);
        fun("spd_lmt_id02", spd_lmt_id02);
        fun("control_way_point_speed_limit02", control_way_point_speed_limit02);
        fun("control_way_point_offset02", control_way_point_offset02);
        fun("reminder_way_point_targe_road_type", reminder_way_point_targe_road_type);
        fun("reminder_way_point_offset", reminder_way_point_offset);
        fun("remain_distance_major_path", remain_distance_major_path);
        fun("remain_distance_major_path_type", remain_distance_major_path_type);
        fun("camr_line_mapping", camr_line_mapping);
        fun("location_qulity", location_qulity);
        fun("enter_ramp_dist", enter_ramp_dist);
        fun("merge_main_road_dist", merge_main_road_dist);
        fun("inavlts", inavlts);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("rm_header", rm_header);
        fun("rm_crrdr_id", rm_crrdr_id);
        fun("rm_featuretag", rm_featuretag);
        fun("rm_dsblrsn", rm_dsblrsn);
        fun("rm_ilc_target", rm_ilc_target);
        fun("left_line", left_line);
        fun("right_line", right_line);
        fun("left_lane", left_lane);
        fun("ego_lane", ego_lane);
        fun("right_lane", right_lane);
        fun("left_roadedge", left_roadedge);
        fun("right_roadedge", right_roadedge);
        fun("float_reserve", float_reserve);
        fun("double_reserve", double_reserve);
        fun("rm_lne_crss", rm_lne_crss);
        fun("rm_lanewidth", rm_lanewidth);
        fun("control_way_point_target_road_type", control_way_point_target_road_type);
        fun("spd_lmt_id01", spd_lmt_id01);
        fun("control_way_point_speed_limit01", control_way_point_speed_limit01);
        fun("control_way_point_offset01", control_way_point_offset01);
        fun("spd_lmt_id02", spd_lmt_id02);
        fun("control_way_point_speed_limit02", control_way_point_speed_limit02);
        fun("control_way_point_offset02", control_way_point_offset02);
        fun("reminder_way_point_targe_road_type", reminder_way_point_targe_road_type);
        fun("reminder_way_point_offset", reminder_way_point_offset);
        fun("remain_distance_major_path", remain_distance_major_path);
        fun("remain_distance_major_path_type", remain_distance_major_path_type);
        fun("camr_line_mapping", camr_line_mapping);
        fun("location_qulity", location_qulity);
        fun("enter_ramp_dist", enter_ramp_dist);
        fun("merge_main_road_dist", merge_main_road_dist);
        fun("inavlts", inavlts);
    }

    friend bool operator==(const ::dpc::RMSMMessage& lhs, const ::dpc::RMSMMessage& rhs) noexcept
    {
        return (lhs.rm_header == rhs.rm_header) && (lhs.rm_crrdr_id == rhs.rm_crrdr_id) && (lhs.rm_featuretag == rhs.rm_featuretag) && (lhs.rm_dsblrsn == rhs.rm_dsblrsn) && (lhs.rm_ilc_target == rhs.rm_ilc_target) && (lhs.left_line == rhs.left_line) && (lhs.right_line == rhs.right_line) && (lhs.left_lane == rhs.left_lane) && (lhs.ego_lane == rhs.ego_lane) && (lhs.right_lane == rhs.right_lane) && (lhs.left_roadedge == rhs.left_roadedge) && (lhs.right_roadedge == rhs.right_roadedge) && (lhs.float_reserve == rhs.float_reserve) && (lhs.double_reserve == rhs.double_reserve) && (lhs.rm_lne_crss == rhs.rm_lne_crss) && (fabs(static_cast<double>(lhs.rm_lanewidth - rhs.rm_lanewidth)) < DBL_EPSILON) && (lhs.control_way_point_target_road_type == rhs.control_way_point_target_road_type) && (lhs.spd_lmt_id01 == rhs.spd_lmt_id01) && (lhs.control_way_point_speed_limit01 == rhs.control_way_point_speed_limit01) && (fabs(static_cast<double>(lhs.control_way_point_offset01 - rhs.control_way_point_offset01)) < DBL_EPSILON) && (lhs.spd_lmt_id02 == rhs.spd_lmt_id02) && (lhs.control_way_point_speed_limit02 == rhs.control_way_point_speed_limit02) && (fabs(static_cast<double>(lhs.control_way_point_offset02 - rhs.control_way_point_offset02)) < DBL_EPSILON) && (lhs.reminder_way_point_targe_road_type == rhs.reminder_way_point_targe_road_type) && (fabs(static_cast<double>(lhs.reminder_way_point_offset - rhs.reminder_way_point_offset)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.remain_distance_major_path - rhs.remain_distance_major_path)) < DBL_EPSILON) && (lhs.remain_distance_major_path_type == rhs.remain_distance_major_path_type) && (lhs.camr_line_mapping == rhs.camr_line_mapping) && (lhs.location_qulity == rhs.location_qulity) && (fabs(static_cast<double>(lhs.enter_ramp_dist - rhs.enter_ramp_dist)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.merge_main_road_dist - rhs.merge_main_road_dist)) < DBL_EPSILON) && (lhs.inavlts == rhs.inavlts);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_RMSMMESSAGE_H
